@@ -13,6 +13,7 @@ import bz2
 import lzma
 import zlib
 
+from .const import NAMED_TYPES
 from .io.binary_encoder import BinaryEncoder
 from .io.json_encoder import AvroJSONEncoder
 from .validation import _validate
@@ -140,7 +141,7 @@ def write_union(encoder, datum, schema, named_schemas, fname):
         (name, datum) = datum
         for index, candidate in enumerate(schema):
             extracted_type = extract_record_type(candidate)
-            if extracted_type == "record":
+            if extracted_type in NAMED_TYPES:
                 schema_name = candidate["name"]
             else:
                 schema_name = extracted_type
@@ -185,6 +186,12 @@ def write_union(encoder, datum, schema, named_schemas, fname):
             if _validate(datum, candidate, named_schemas, raise_errors=False):
                 record_type = extract_record_type(candidate)
                 if record_type == "record":
+                    logical_type = extract_logical_type(candidate)
+                    if logical_type:
+                        prepare = LOGICAL_WRITERS.get(logical_type)
+                        if prepare:
+                            datum = prepare(datum, candidate)
+
                     candidate_fields = set(f["name"] for f in candidate["fields"])
                     datum_fields = set(datum)
                     fields = len(candidate_fields.intersection(datum_fields))
@@ -391,7 +398,7 @@ else:
 class GenericWriter:
     def __init__(self, schema, metadata=None, validator=None):
         self._named_schemas = {}
-        self.schema = parse_schema(schema, _named_schemas=self._named_schemas)
+        self.schema = parse_schema(schema, self._named_schemas)
         self.validate_fn = _validate if validator is True else validator
         self.metadata = metadata or {}
 
@@ -674,7 +681,7 @@ def schemaless_writer(fo, schema, record):
     Note: The ``schemaless_writer`` can only write a single record.
     """
     named_schemas = {}
-    schema = parse_schema(schema, _named_schemas=named_schemas)
+    schema = parse_schema(schema, named_schemas)
 
     encoder = BinaryEncoder(fo)
     write_data(encoder, record, schema, named_schemas, "")
